using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Kerr.WindowClippings.FilterWatermark
{
    public partial class FilterWatermarkWindow : Form
    {
        public FilterWatermarkWindow()
        {
            InitializeComponent();

            MinimumSize = Size;

            m_topLeft.Tag = AnchorStyles.Top | AnchorStyles.Left;
            m_top.Tag = AnchorStyles.Top;
            m_topRight.Tag = AnchorStyles.Top | AnchorStyles.Right;
            m_left.Tag = AnchorStyles.Left;
            m_center.Tag = AnchorStyles.None;
            m_right.Tag = AnchorStyles.Right;
            m_bottomLeft.Tag = AnchorStyles.Bottom | AnchorStyles.Left;
            m_bottom.Tag = AnchorStyles.Bottom;
            m_bottomRight.Tag = AnchorStyles.Bottom | AnchorStyles.Right;

            m_brush = new HatchBrush(HatchStyle.LargeCheckerBoard,
                                     Color.White,
                                     Color.FromArgb(204, 204, 204));
        }

        public string ImagePath
        {
            get
            {
                return m_imagePath.Text;
            }
            set
            {
                m_imagePath.Text = value;
            }
        }

        public AnchorStyles Alignment
        {
            get
            {
                return m_alignment;
            }
            set
            {
                foreach (RadioButton button in m_alignmentGroup.Controls)
                {
                    if (value == (AnchorStyles) button.Tag)
                    {
                        button.Checked = true;
                        break;
                    }
                }
            }
        }

        private void AlignmentChangedEventHandler(object sender, 
                                                  EventArgs args)
        {
            RadioButton button = (RadioButton) sender;
            m_alignment = (AnchorStyles)button.Tag;
            RedrawPreview();
        }

        private void BrowseEventHandler(object sender,
                                        EventArgs args)
        {
            using (OpenFileDialog window = new OpenFileDialog())
            {
                window.FileName = m_imagePath.Text;
                window.Filter = "Image files (*.bmp;*.jpg;*.png;*.tif)|*.bmp;*.jpg;*.png;*.tif|All files (*.*)|*.*";

                if (DialogResult.OK == window.ShowDialog(this))
                {
                    m_imagePath.Text = window.FileName;
                }
            }
        }

        private void ImagePathChangedEventHandler(object sender,
                                                  EventArgs args)
        {
            if (null != m_bitmap)
            {
                m_bitmap.Dispose();
                m_bitmap = null;
            }

            if (File.Exists(m_imagePath.Text))
            {
                try
                {
                    m_bitmap = new Bitmap(m_imagePath.Text);
                }
                catch (Exception e)
                {
                    // The user probably didn't pick a supported image file...

                    Debug.WriteLine("ImagePathChangedEventHandler exception ignored: " + e.Message);
                }
            }

            RedrawPreview();
        }

        private void ResizeEventHandler(object sender,
                                        EventArgs args)
        {
            RedrawPreview();
        }

        private void PaintEventHandler(object sender, 
                                       PaintEventArgs args)
        {
            RedrawPreview(args.Graphics);
        }

        private void RedrawPreview()
        {
            using (Graphics graphics = Graphics.FromHwnd(m_previewPanel.Handle))
            {
                RedrawPreview(graphics);
            }
        }

        private void RedrawPreview(Graphics graphics)
        {
            if (null == m_bitmap)
            {
                m_bitmap = Bitmap.FromHicon(LoadImage(IntPtr.Zero,
                                                      new IntPtr(IDI_HAND),
                                                      IMAGE_ICON,
                                                      32,
                                                      32,
                                                      LR_SHARED));
            }

            graphics.FillRectangle(m_brush,
                                   0,
                                   0,
                                   m_previewPanel.Width,
                                   m_previewPanel.Height);

            Helpers.DrawWatermark(graphics,
                                  m_previewPanel.Width,
                                  m_previewPanel.Height,
                                  m_bitmap,
                                  m_alignment);
        }

        [DllImport("user32.dll")]
        private static extern IntPtr LoadImage(IntPtr instance,
                                               IntPtr name,
                                               uint type,
                                               int width,
                                               int height,
                                               uint flags);

        private const uint IMAGE_ICON = 1;
        private const uint LR_SHARED = 0x00008000;
        private const uint IDI_HAND = 32513;

        private Bitmap m_bitmap;
        private Brush m_brush;
        private AnchorStyles m_alignment;
    }
}